
! Copyright (C) 2016 J. K. Dewhurst, S. Sharma and E. K. U. Gross.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

subroutine eveqnu
use modmain
implicit none
! local variables
integer ist,i,j

! loop over second-variational states
do ist=1,nstsv
! loop over kappa-points
  do j=1,nkpa

  end do
end do


return
end subroutine

