
! Copyright (C) 2015 J. K. Dewhurst, S. Sharma and E. K. U. Gross.
! This file is distributed under the terms of the GNU General Public License.
! See the file COPYING for license details.

module modgw

! number of complex frequencies to be used for the GW calculation
integer nwgw
! the complex GW frequencies
complex(8), allocatable :: wgw(:)

end module

